/****************************************************************************
** Meta object code from reading C++ file 'schedulemessage.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kcalendarcore-6.29.0/src/schedulemessage.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'schedulemessage.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13KCalendarCore15ScheduleMessageE_t {};
} // unnamed namespace

template <> constexpr inline auto KCalendarCore::ScheduleMessage::qt_create_metaobjectdata<qt_meta_tag_ZN13KCalendarCore15ScheduleMessageE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KCalendarCore::ScheduleMessage",
        "Status",
        "PublishNew",
        "PublishUpdate",
        "Obsolete",
        "RequestNew",
        "RequestUpdate",
        "Unknown"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Status'
        QtMocHelpers::EnumData<enum Status>(1, 1, QMC::EnumFlags{}).add({
            {    2, Status::PublishNew },
            {    3, Status::PublishUpdate },
            {    4, Status::Obsolete },
            {    5, Status::RequestNew },
            {    6, Status::RequestUpdate },
            {    7, Status::Unknown },
        }),
    };
    return QtMocHelpers::metaObjectData<ScheduleMessage, qt_meta_tag_ZN13KCalendarCore15ScheduleMessageE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KCalendarCore::ScheduleMessage::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KCalendarCore15ScheduleMessageE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KCalendarCore15ScheduleMessageE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13KCalendarCore15ScheduleMessageE_t>.metaTypes,
    nullptr
} };

QT_WARNING_POP
