/****************************************************************************
** Meta object code from reading C++ file 'types.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kcalendarcore-6.29.0/src/qml/types.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'types.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN24CalendarListModelForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto CalendarListModelForeign::qt_create_metaobjectdata<qt_meta_tag_ZN24CalendarListModelForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CalendarListModelForeign",
        "QML.Element",
        "CalendarListModel",
        "QML.Foreign",
        "KCalendarCore::CalendarListModel"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<CalendarListModelForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject CalendarListModelForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24CalendarListModelForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24CalendarListModelForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN24CalendarListModelForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN20KCalendarCoreForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto KCalendarCoreForeign::qt_create_metaobjectdata<qt_meta_tag_ZN20KCalendarCoreForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KCalendarCoreForeign",
        "QML.Element",
        "KCalendarCore",
        "QML.Foreign",
        "QML.ForeignIsNamespace",
        "true"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    2 },
            {    4,    5 },
    });
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN20KCalendarCoreForeignE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}

static constexpr auto qt_staticMetaObjectContent_ZN20KCalendarCoreForeignE =
    KCalendarCoreForeign::qt_create_metaobjectdata<qt_meta_tag_ZN20KCalendarCoreForeignE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN20KCalendarCoreForeignE =
    qt_staticMetaObjectContent_ZN20KCalendarCoreForeignE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN20KCalendarCoreForeignE =
    qt_staticMetaObjectContent_ZN20KCalendarCoreForeignE.relocatingData;

Q_CONSTINIT const QMetaObject KCalendarCoreForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN20KCalendarCoreForeignE.stringdata,
    qt_staticMetaObjectStaticContent_ZN20KCalendarCoreForeignE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN20KCalendarCoreForeignE.metaTypes,
    nullptr
} };

QT_WARNING_POP
